"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk = require("chalk");
const moment = require("moment");
const path = require("path");
const ctircLoader_1 = require("../options/ctircLoader");
const CTILogger_1 = require("../tools/CTILogger");
const CTIUtility_1 = require("../tools/CTIUtility");
const CommandModule_1 = require("./CommandModule");
const { isNotEmpty } = CTIUtility_1.CTIUtility;
class EntrypointCommandModule {
    do(cliCwd, passed) {
        return __awaiter(this, void 0, void 0, function* () {
            const cwd = isNotEmpty(passed.globOptions) && isNotEmpty(passed.globOptions.cwd)
                ? passed.globOptions.cwd
                : process.cwd();
            const { readedFrom, option } = ctircLoader_1.ctircLoader({
                cwd: cliCwd,
                fromClioption: passed,
                inputDir: cwd,
            });
            const logger = new CTILogger_1.CTILogger(option.verbose);
            logger.log('configuration from: ', readedFrom === '' ? 'default' : readedFrom);
            try {
                logger.log(chalk.default.yellowBright('Option: '), option);
                const targetFileGlob = option.targetExts.map((ext) => `*.${ext}`).join('|');
                const allTsFiles = yield CommandModule_1.CommandModule.promisify.glob(`**/+(${targetFileGlob})`, option.globOptions);
                const tsFiles = CommandModule_1.CommandModule.targetFileFilter({
                    logger,
                    option,
                    filenames: allTsFiles,
                });
                const dupLibDirs = tsFiles
                    .filter((tsFile) => tsFile.split('/').length > 1)
                    .map((tsFile) => {
                    const splitted = tsFile.split('/');
                    const allPath = Array(splitted.length - 1)
                        .fill(0)
                        .map((_, index) => index + 1)
                        .map((index) => {
                        const a = splitted.slice(0, index).join('/');
                        return a;
                    });
                    return allPath;
                })
                    .reduce((aggregated, libPath) => {
                    return aggregated.concat(libPath);
                }, []);
                const dirSet = new Set();
                dupLibDirs.forEach((dir) => dirSet.add(dir));
                tsFiles.map((tsFile) => path.dirname(tsFile)).forEach((dir) => dirSet.add(dir));
                const tsDirs = Array.from(dirSet);
                if (option.includeCWD &&
                    tsDirs.findIndex((dir) => path.resolve(dir) === path.resolve('.')) < 0) {
                    tsDirs.push('.');
                }
                tsDirs.sort((left, right) => {
                    const llen = left.split('/').length;
                    const rlen = right.split('/').length;
                    if (llen > rlen) {
                        return -1;
                    }
                    if (llen < rlen) {
                        return 1;
                    }
                    return 0;
                });
                yield this.write({ logger, option, directories: tsDirs });
                logger.flog(chalk.default.green(`entrypoint create succeeded: ${option.globOptions.cwd}`));
            }
            catch (err) {
                logger.ferror(chalk.default.redBright(err));
            }
        });
    }
    write({ directories, option, logger, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const indexFiles = option.targetExts.map((targetExt) => `entrypoint.${targetExt}`);
            try {
                const zipFiles = yield Promise.all(directories.map((directory) => {
                    return (() => __awaiter(this, void 0, void 0, function* () {
                        const resolvePath = path.resolve(option.globOptions.cwd || __dirname);
                        const elements = yield CommandModule_1.CommandModule.promisify.readDir(path.join(resolvePath, directory));
                        const targets = elements
                            .filter((element) => indexFiles.indexOf(element) < 0)
                            .filter((element) => {
                            const isTarget = option.targetExts.reduce((result, ext) => {
                                return result || CTIUtility_1.CTIUtility.addDot(ext) === path.extname(element);
                            }, false);
                            const isHaveTarget = directories.indexOf(path.join(directory, element)) >= 0;
                            return isTarget || isHaveTarget;
                        });
                        const stats = yield Promise.all(targets.map((target) => CommandModule_1.CommandModule.promisify.stat(path.join(resolvePath, directory, target))));
                        const categorized = targets.reduce((result, target, index) => {
                            if (stats[index].isDirectory()) {
                                result.dir.push(target);
                            }
                            else {
                                result.allFiles.push(target);
                            }
                            return result;
                        }, { dir: [], allFiles: [] });
                        categorized.dir.sort();
                        categorized.allFiles = CommandModule_1.CommandModule.targetFileFilter({
                            logger,
                            option,
                            filenames: categorized.allFiles,
                        });
                        const filesInDirectory = categorized.allFiles.filter((element) => {
                            return !option.fileExcludePatterns.reduce((result, excludePattern) => {
                                return result || element.indexOf(excludePattern) >= 0;
                            }, false);
                        });
                        filesInDirectory.sort();
                        return filesInDirectory.map((file) => path.relative(resolvePath, path.join(resolvePath, directory, file)));
                    }))();
                }));
                const files = zipFiles.reduce((aggregated, _files) => {
                    return aggregated.concat(_files);
                });
                const exportString = files.map((target) => {
                    let targetFileWithoutExt = target;
                    option.targetExts.forEach((ext) => {
                        return (targetFileWithoutExt = targetFileWithoutExt.replace(CTIUtility_1.CTIUtility.addDot(ext), ''));
                    });
                    logger.log(chalk.default.green('entrypoint added from:'), target);
                    if (option.useSemicolon) {
                        return `export * from ${option.quote}./${targetFileWithoutExt}${option.quote};`;
                    }
                    return `export * from ${option.quote}./${targetFileWithoutExt}${option.quote}`;
                });
                const comment = (() => {
                    if (option.useTimestamp) {
                        return `// created from ${option.quote}create-ts-index${option.quote} ${moment(new Date()).format('YYYY-MM-DD HH:mm')}\n\n`;
                    }
                    return `// created from ${option.quote}create-ts-index${option.quote}\n\n`;
                })();
                const sortedExportString = exportString.sort();
                const fileContent = comment + CTIUtility_1.CTIUtility.addNewline(option, sortedExportString.join('\n'));
                const cwdPath = option.globOptions.cwd || __dirname;
                logger.log(chalk.default.green('entrypoiny writed:', `${cwdPath}${path.sep}entrypoint.ts`));
                yield CommandModule_1.CommandModule.promisify.writeFile(path.join(cwdPath, 'entrypoint.ts'), fileContent, 'utf8');
            }
            catch (err) {
                logger.error(chalk.default.red('indexWriter: ', err.message));
                logger.error(chalk.default.red('indexWriter: ', err.stack));
            }
        });
    }
}
exports.EntrypointCommandModule = EntrypointCommandModule;
//# sourceMappingURL=EntrypointCommandModule.js.map