"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const CTIUtility_1 = require("../tools/CTIUtility");
const { isNotEmpty } = CTIUtility_1.CTIUtility;
class CreateTsIndexOption {
    static factory({ option, partialOption, }) {
        if (isNotEmpty(option)) {
            const createTsIndexOption = new CreateTsIndexOption(option);
            return createTsIndexOption;
        }
        if (isNotEmpty(partialOption)) {
            const createTsIndexOption = CreateTsIndexOption.getOption(partialOption);
            return createTsIndexOption;
        }
        const defaultOption = CreateTsIndexOption.getDefailtICreateTsIndexOption();
        return new CreateTsIndexOption(defaultOption);
    }
    static cliOptionBuilder(args, cwd) {
        const options = {
            addNewline: args.addnewline,
            excludes: args.excludes,
            fileExcludePatterns: args.fileexcludes,
            fileFirst: args.filefirst,
            globOptions: {
                cwd,
            },
            includeCWD: args.includecwd,
            quote: args.quote,
            targetExts: args.targetexts,
            useSemicolon: args.usesemicolon,
            useTimestamp: args.usetimestamp,
            verbose: args.verbose,
        };
        return options;
    }
    static getDefailtICreateTsIndexOption(cwd) {
        return {
            addNewline: true,
            excludes: ['@types', 'typings', '__test__', '__tests__', 'node_modules'],
            fileExcludePatterns: [],
            fileFirst: false,
            globOptions: {
                cwd: cwd || process.cwd(),
                dot: true,
                nonull: true,
            },
            includeCWD: true,
            quote: "'",
            targetExts: ['ts', 'tsx'],
            useSemicolon: true,
            useTimestamp: false,
            verbose: false,
        };
    }
    static merge(deep, isNotCopyEmpty, src, ...dsts) {
        const merged = (() => {
            if (deep) {
                return Object.assign({}, src);
            }
            return src;
        })();
        const srcKeys = Object.keys(src);
        dsts.forEach((dst) => {
            const dstKeys = Object.keys(dst);
            const keySet = new Set(srcKeys.concat(dstKeys));
            Array.from(keySet).forEach((key) => {
                if (typeof merged[key] === 'string' ||
                    typeof merged[key] === 'number' ||
                    typeof merged[key] === 'boolean' ||
                    typeof merged[key] === 'bigint' ||
                    typeof merged[key] === 'symbol' ||
                    typeof merged[key] === 'undefined') {
                    if (isNotCopyEmpty && (isNotEmpty(dst[key]) && dst[key])) {
                        merged[key] = dst[key];
                    }
                    else if (!isNotCopyEmpty) {
                        merged[key] = dst[key];
                    }
                }
                else if (typeof merged[key] === 'object' && Array.isArray(merged[key])) {
                    const element = new Set(merged[key].concat(dst[key]));
                    merged[key] = Array.from(element);
                }
                else if (typeof merged[key] === 'object') {
                    merged[key] = CreateTsIndexOption.merge(deep, isNotCopyEmpty, merged[key], dst[key]);
                }
            });
        });
        return merged;
    }
    /**
     * Merge configurations by passing order
     *
     * @param passeds Passed variadic configurations
     */
    static mergeOptions(...passeds) {
        const [first, remains] = passeds;
        const merged = CreateTsIndexOption.merge(true, true, first, remains);
        return new CreateTsIndexOption(merged);
    }
    static getOption(passed) {
        const option = CreateTsIndexOption.getDefailtICreateTsIndexOption();
        option.fileFirst = isNotEmpty(passed.fileFirst) ? passed.fileFirst : option.fileFirst;
        option.addNewline = isNotEmpty(passed.addNewline) ? passed.addNewline : option.addNewline;
        option.useSemicolon = isNotEmpty(passed.useSemicolon)
            ? passed.useSemicolon
            : option.useSemicolon;
        option.useTimestamp = isNotEmpty(passed.useTimestamp)
            ? passed.useTimestamp
            : option.useTimestamp;
        option.includeCWD = isNotEmpty(passed.includeCWD) ? passed.includeCWD : option.includeCWD;
        option.fileExcludePatterns = isNotEmpty(passed.fileExcludePatterns)
            ? passed.fileExcludePatterns
            : option.fileExcludePatterns;
        option.excludes = isNotEmpty(passed.excludes) ? passed.excludes : option.excludes;
        option.targetExts = isNotEmpty(passed.targetExts) ? passed.targetExts : option.targetExts;
        // remove sorting
        // option.targetExts = option.targetExts.sort((l, r) => r.length - l.length);
        if (isNotEmpty(passed.globOptions)) {
            option.globOptions.cwd = isNotEmpty(passed.globOptions.cwd)
                ? passed.globOptions.cwd
                : option.globOptions.cwd;
            option.globOptions.nonull = isNotEmpty(passed.globOptions.nonull)
                ? passed.globOptions.nonull
                : option.globOptions.nonull;
            option.globOptions.dot = isNotEmpty(passed.globOptions.dot)
                ? passed.globOptions.dot
                : option.globOptions.dot;
        }
        option.verbose = isNotEmpty(passed.verbose) ? passed.verbose : option.verbose;
        return new CreateTsIndexOption(option);
    }
    constructor(fileFirst, addNewline, useSemicolon, useTimestamp, includeCWD, excludes, fileExcludePatterns, targetExts, globOptions, quote, verbose) {
        if (typeof fileFirst === 'boolean') {
            this.fileFirst = fileFirst;
            this.addNewline = isNotEmpty(addNewline) ? addNewline : true;
            this.useSemicolon = isNotEmpty(useSemicolon) ? useSemicolon : true;
            this.useTimestamp = isNotEmpty(useTimestamp) ? useTimestamp : false;
            this.includeCWD = isNotEmpty(includeCWD) ? includeCWD : true;
            this.excludes = isNotEmpty(excludes)
                ? excludes
                : ['@types', 'typings', '__test__', '__tests__', 'node_modules'];
            this.fileExcludePatterns = isNotEmpty(fileExcludePatterns) ? fileExcludePatterns : [];
            this.targetExts = isNotEmpty(targetExts) ? targetExts : ['ts', 'tsx'];
            this.globOptions = isNotEmpty(globOptions)
                ? globOptions
                : {
                    cwd: process.cwd(),
                    dot: true,
                    nonull: true,
                };
            this.quote = isNotEmpty(quote) ? quote : "'";
            this.verbose = isNotEmpty(verbose) ? verbose : false;
        }
        else {
            const { fileFirst: _fileFirst, addNewline: _addNewline, useSemicolon: _useSemicolon, useTimestamp: _useTimestamp, includeCWD: _includeCWD, excludes: _excludes, fileExcludePatterns: _fileExcludePatterns, targetExts: _targetExts, globOptions: _globOptions, quote: _quote, verbose: _verbose, } = fileFirst || CreateTsIndexOption.getDefailtICreateTsIndexOption();
            this.fileFirst = _fileFirst;
            this.addNewline = _addNewline;
            this.useSemicolon = _useSemicolon;
            this.useTimestamp = _useTimestamp;
            this.includeCWD = _includeCWD;
            this.excludes = _excludes;
            this.fileExcludePatterns = _fileExcludePatterns;
            this.targetExts = _targetExts;
            this.globOptions = _globOptions;
            this.quote = _quote;
            this.verbose = _verbose;
        }
    }
}
exports.CreateTsIndexOption = CreateTsIndexOption;
//# sourceMappingURL=CreateTsIndexOption.js.map