"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable no-console
const debug_1 = require("debug");
const fs = require("fs");
const json5 = require("json5");
const path = require("path");
const CreateTsIndexOption_1 = require("./CreateTsIndexOption");
const log = debug_1.default('cti:ctircLoader');
const CTIRC_FILENAME = '.ctirc';
function ctircLoader({ inputDir, cwd, fromClioption, }) {
    log(`inputDir: (${inputDir}) /cwd: (${cwd})`);
    // Step01. Read configuration file on parametered input directory
    const targetDirs = (() => {
        const _target = [];
        if (fs.existsSync(inputDir) && fs.existsSync(path.join(inputDir, CTIRC_FILENAME))) {
            _target.push(inputDir);
        }
        if (fs.existsSync(cwd) && fs.existsSync(path.join(cwd, CTIRC_FILENAME))) {
            _target.push(cwd);
        }
        return _target;
    })();
    log('targetDirs: ', targetDirs);
    // Stop02. Load configuration from .ctirc
    try {
        if (targetDirs.length <= 0) {
            return {
                option: new CreateTsIndexOption_1.CreateTsIndexOption(CreateTsIndexOption_1.CreateTsIndexOption.getOption(fromClioption)),
                readedFrom: 'from cli option',
            };
        }
        const fromConfigfiles = targetDirs.map((targetDir) => {
            const ctircPath = path.join(targetDir, CTIRC_FILENAME);
            const rawConf = fs.readFileSync(ctircPath).toString();
            const fromConfigfile = Object.assign({}, json5.parse(rawConf), { globOptions: { cwd: inputDir } });
            return fromConfigfile;
        });
        const mergedCTIRC = (() => {
            if (fromConfigfiles.length > 2) {
                const [soruce, dsts] = fromConfigfiles;
                const _mergedCTIRC = CreateTsIndexOption_1.CreateTsIndexOption.merge(true, true, soruce, dsts);
                return _mergedCTIRC;
            }
            const [_firstCTIRC] = fromConfigfiles;
            return _firstCTIRC;
        })();
        log('mergedCTIRC: ', mergedCTIRC);
        mergedCTIRC.__for_debug_from = 'from-config-file';
        fromClioption.__for_debug_from = 'from-cli-option';
        const option = CreateTsIndexOption_1.CreateTsIndexOption.mergeOptions(CreateTsIndexOption_1.CreateTsIndexOption.getDefailtICreateTsIndexOption(), mergedCTIRC, fromClioption);
        return { option, readedFrom: 'from config file' };
    }
    catch (err) {
        log(err.message);
        log(err.stacktrace);
        const option = new CreateTsIndexOption_1.CreateTsIndexOption();
        return { option, readedFrom: '' };
    }
}
exports.ctircLoader = ctircLoader;
//# sourceMappingURL=ctircLoader.js.map